import { isBrowser, log } from './misc/Util';
/**
 * Wrapper class for `RTCDataChannel` and `WebSocket`.
 */
var Channel = /** @class */ (function () {
    /**
     * Creates a channel from existing `RTCDataChannel` or `WebSocket`.
     */
    function Channel(wc, connection, options) {
        if (options === void 0) { options = { id: 1 }; }
        var _this = this;
        log.channel("New Channel: Me: " + wc.myId + " with " + options.id);
        this.wc = wc;
        this.connection = connection;
        this._id = options.id;
        this.rtcPeerConnection = options.rtcPeerConnection;
        this.isIntermediary = false;
        this.missedHeartbeat = 0;
        this.updateHeartbeatMsg(this.wc.topologyService.heartbeat);
        if (this.rtcPeerConnection) {
            this.maximumMissedHeartBeat = 3;
        }
        else {
            this.maximumMissedHeartBeat = 5;
        }
        // Configure `send` function
        if (isBrowser) {
            connection.binaryType = 'arraybuffer';
            this.send = this.sendInBrowser;
        }
        else if (!this.rtcPeerConnection) {
            this.send = this.sendInNodeViaWebSocket;
        }
        else {
            connection.binaryType = 'arraybuffer';
            this.send = this.sendInNodeViaDataChannel;
        }
        // Configure handlers
        this.connection.onmessage = function (_a) {
            var data = _a.data;
            return wc.onMessageProxy(_this, new Uint8Array(data));
        };
        this.connection.onclose = function (evt) {
            log.channel("Connection with " + _this.id + " has closed");
            wc.topologyService.onChannelClose(evt, _this);
        };
        this.connection.onerror = function (evt) {
            log.channel('Channel error: ', evt);
            wc.topologyService.onChannelError(evt, _this);
            _this.close();
        };
    }
    Object.defineProperty(Channel.prototype, "id", {
        get: function () {
            return this._id;
        },
        set: function (value) {
            this._id = value;
            this.fullHeartbeatMsg = this.wc.encode({ recipientId: value, content: this.topologyHeartbeatMsg });
        },
        enumerable: true,
        configurable: true
    });
    Channel.prototype.close = function () {
        this.connection.close();
        if (this.rtcPeerConnection) {
            this.rtcPeerConnection.close();
        }
    };
    Channel.prototype.closeQuietly = function () {
        this.connection.onmessage = undefined;
        this.connection.onclose = undefined;
        this.connection.onerror = undefined;
        this.close();
    };
    Channel.prototype.sendHeartbeat = function () {
        this.send(this.fullHeartbeatMsg);
    };
    Channel.prototype.updateHeartbeatMsg = function (heartbeatMsg) {
        this.topologyHeartbeatMsg = heartbeatMsg;
        this.fullHeartbeatMsg = this.wc.encode({ recipientId: this._id, content: heartbeatMsg });
    };
    Channel.prototype.sendInBrowser = function (data) {
        try {
            this.connection.send(data);
        }
        catch (err) {
            log.channel('Channel sendInBrowser ERROR', err);
        }
    };
    Channel.prototype.sendInNodeViaWebSocket = function (data) {
        try {
            this.connection.send(data, { binary: true });
        }
        catch (err) {
            log.channel('Channel sendInNodeViaWebSocket ERROR', err);
        }
    };
    Channel.prototype.sendInNodeViaDataChannel = function (data) {
        this.sendInBrowser(data.slice(0));
    };
    return Channel;
}());
export { Channel };
