import { filter, pluck } from 'rxjs/operators';
import { Subject } from 'rxjs/Subject';
import { log } from './misc/Util';
import { signaling as sigProto } from './proto';
var MAXIMUM_MISSED_HEARTBEAT = 3;
var HEARTBEAT_INTERVAL = 5000;
/* WebSocket error codes */
var HEARTBEAT_ERROR_CODE = 4002;
var MESSAGE_ERROR_CODE = 4010;
/* Preconstructed messages */
var heartbeatMsg = sigProto.Message.encode(sigProto.Message.create({ heartbeat: true })).finish();
export var SignalingState;
(function (SignalingState) {
    SignalingState[SignalingState["CONNECTING"] = 0] = "CONNECTING";
    SignalingState[SignalingState["CONNECTED"] = 1] = "CONNECTED";
    SignalingState[SignalingState["STABLE"] = 2] = "STABLE";
    SignalingState[SignalingState["CLOSING"] = 3] = "CLOSING";
    SignalingState[SignalingState["CLOSED"] = 4] = "CLOSED";
})(SignalingState || (SignalingState = {}));
/**
 * This class represents a door of the `WebChannel` for the current peer. If the door
 * is open, then clients can join the `WebChannel` through this peer. There are as
 * many doors as peers in the `WebChannel` and each of them can be closed or opened.
 */
var Signaling = /** @class */ (function () {
    function Signaling(wc, url) {
        // public
        this.url = url;
        this.state = SignalingState.CLOSED;
        // private
        this.wc = wc;
        this.completeWs = function () { };
        this.stateSubject = new Subject();
        this.channelSubject = new Subject();
    }
    Object.defineProperty(Signaling.prototype, "onState", {
        get: function () {
            return this.stateSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Signaling.prototype, "onChannel", {
        get: function () {
            return this.channelSubject.asObservable();
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Notify Signaling server that you had joined the network and ready
     * to join new peers to the network.
     */
    Signaling.prototype.open = function () {
        if (this.state === SignalingState.CONNECTED) {
            this.send({ stable: true });
            this.setState(SignalingState.STABLE);
        }
    };
    Signaling.prototype.join = function (key) {
        var _this = this;
        if (this.state !== SignalingState.CLOSING && this.state !== SignalingState.CLOSED) {
            this.ws.onclose = function () { };
            this.ws.onmessage = function () { };
            this.ws.onerror = function () { };
            clearInterval(this.heartbeatInterval);
            this.missedHeartbeat = 0;
            this.completeWs();
            this.ws.close();
        }
        this.setState(SignalingState.CONNECTING);
        this.wc.webSocketBuilder
            .connect(this.getFullURL(key))
            .then(function (ws) {
            _this.ws = ws;
            _this.wsObservable = _this.createObservable(_this.ws);
            _this.startHeartbeat();
            _this.wsObservable.subscribe(function (msg) {
                switch (msg.type) {
                    case 'heartbeat':
                        _this.missedHeartbeat = 0;
                        break;
                    case 'isFirst':
                        if (msg.isFirst) {
                            _this.setState(SignalingState.STABLE);
                        }
                        else {
                            _this.connectOverSignaling();
                        }
                        break;
                }
            });
        })
            .catch(function () { return _this.setState(SignalingState.CLOSED); });
    };
    /**
     * Close the `WebSocket` with Signaling server.
     */
    Signaling.prototype.close = function () {
        if (this.state !== SignalingState.CLOSING && this.state !== SignalingState.CLOSED) {
            this.setState(SignalingState.CLOSING);
            if (this.ws) {
                this.ws.close(1000);
            }
        }
    };
    Signaling.prototype.connectOverSignaling = function () {
        var _this = this;
        if (this.ws.readyState === WebSocket.OPEN) {
            this.wc.webRTCBuilder.connectOverSignaling(this.getWebRTCStream())
                .then(function () { return _this.setState(SignalingState.CONNECTED); })
                .catch(function () { return _this.send({ tryAnother: true }); });
        }
    };
    Signaling.prototype.setState = function (state) {
        var _this = this;
        if (this.state !== state) {
            this.state = state;
            this.stateSubject.next(state);
            if (state === SignalingState.STABLE) {
                this.wc.webRTCBuilder
                    .onChannelFromSignaling(this.getWebRTCStream())
                    .subscribe(function (ch) { return _this.channelSubject.next(ch); });
            }
        }
    };
    Signaling.prototype.startHeartbeat = function () {
        var _this = this;
        this.missedHeartbeat = 0;
        this.heartbeatInterval = global.setInterval(function () {
            try {
                _this.missedHeartbeat++;
                if (_this.missedHeartbeat >= MAXIMUM_MISSED_HEARTBEAT) {
                    throw new Error('Too many missed heartbeats');
                }
                _this.heartbeat();
            }
            catch (err) {
                global.clearInterval(_this.heartbeatInterval);
                log.signaling('Closing connection with Signaling. Reason: ' + err.message);
                _this.setState(SignalingState.CLOSING);
                _this.ws.close(HEARTBEAT_ERROR_CODE, 'Signaling is not responding');
            }
        }, HEARTBEAT_INTERVAL);
    };
    Signaling.prototype.createObservable = function (ws) {
        var _this = this;
        var subject = new Subject();
        this.completeWs = function () { return subject.complete(); };
        ws.binaryType = 'arraybuffer';
        ws.onmessage = function (evt) {
            try {
                subject.next(sigProto.Message.decode(new Uint8Array(evt.data)));
            }
            catch (err) {
                ws.close(MESSAGE_ERROR_CODE, err.message);
            }
        };
        ws.onerror = function (err) { return subject.error(err); };
        ws.onclose = function (closeEvt) {
            clearInterval(_this.heartbeatInterval);
            _this.missedHeartbeat = 0;
            _this.setState(SignalingState.CLOSED);
            subject.complete();
            log.signaling("Connection with Signaling '" + _this.url + "' closed: " + closeEvt.code + ": " + closeEvt.reason);
        };
        return subject.asObservable();
    };
    Signaling.prototype.send = function (msg) {
        if (this.ws.readyState === WebSocket.OPEN) {
            try {
                this.ws.send(sigProto.Message.encode(sigProto.Message.create(msg)).finish());
            }
            catch (err) {
                log.signaling('Failed send to Signaling: ' + err.message);
            }
        }
    };
    Signaling.prototype.heartbeat = function () {
        if (this.ws.readyState === WebSocket.OPEN) {
            try {
                this.ws.send(heartbeatMsg);
            }
            catch (err) {
                log.signaling('Failed send to Signaling: ' + err.message);
            }
        }
    };
    Signaling.prototype.getWebRTCStream = function () {
        var _this = this;
        return {
            message: this.wsObservable
                .pipe(filter(function (_a) {
                var type = _a.type;
                return type === 'content';
            }), pluck('content')),
            send: function (m) { return _this.send({ content: m }); },
        };
    };
    Signaling.prototype.getFullURL = function (params) {
        if (this.url.endsWith('/')) {
            return this.url + params;
        }
        else {
            return this.url + '/' + params;
        }
    };
    return Signaling;
}());
export { Signaling };
