var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
import { user } from './proto';
/**
 * Maximum size of the user message sent over `Channel` (without metadata).
 */
var MAX_USER_MSG_SIZE = 15000;
/**
 * Maximum message id number.
 */
var MAX_MSG_ID_SIZE = 65535;
var textEncoder = new global.TextEncoder();
var textDecoder = new global.TextDecoder();
/**
 * Message builder service is responsible to build messages to send them over the
 * `WebChannel` and treat messages received by the `WebChannel`. It also manage
 * big messages (more then 16ko) sent by users. Internal messages are always less
 * 16ko.
 */
var UserMessage = /** @class */ (function () {
    function UserMessage() {
        this.buffers = new Map();
    }
    /**
     * Encode user message for sending over the network.
     */
    UserMessage.prototype.encode = function (data) {
        var _a = this.userDataToType(data), type = _a.type, bytes = _a.bytes;
        var msg = { length: bytes.length, type: type };
        if (bytes.length <= MAX_USER_MSG_SIZE) {
            msg.full = bytes;
            return [user.Message.encode(user.Message.create(msg)).finish()];
        }
        else {
            msg.chunk = { id: Math.ceil(Math.random() * MAX_MSG_ID_SIZE) };
            var numberOfChunks = Math.ceil(bytes.length / MAX_USER_MSG_SIZE);
            var res = new Array(numberOfChunks);
            for (var i = 0; i < numberOfChunks; i++) {
                var length_1 = Math.min(MAX_USER_MSG_SIZE, bytes.length - MAX_USER_MSG_SIZE * i);
                var begin = MAX_USER_MSG_SIZE * i;
                var end = begin + length_1;
                msg.chunk.number = i;
                msg.chunk.content = new Uint8Array(bytes.slice(begin, end));
                res[i] = user.Message.encode(user.Message.create(msg)).finish();
            }
            return res;
        }
    };
    /**
     * Decode user message received from the network.
     */
    UserMessage.prototype.decode = function (bytes, senderId) {
        var msg = user.Message.decode(bytes);
        var content;
        switch (msg.content) {
            case 'full': {
                content = msg.full;
                break;
            }
            case 'chunk': {
                var buffer = this.getBuffer(senderId, msg.chunk.id);
                if (buffer === undefined) {
                    buffer = new Buffer(msg.length, msg.chunk.content, msg.chunk.number);
                    this.setBuffer(senderId, msg.chunk.id, buffer);
                    content = undefined;
                }
                else {
                    content = buffer.append(msg.chunk.content, msg.chunk.number);
                }
                break;
            }
            default: {
                throw new Error('Unknown message integrity');
            }
        }
        if (content !== undefined) {
            switch (msg.type) {
                case user.Message.Type.U_INT_8_ARRAY:
                    return content;
                case user.Message.Type.STRING:
                    return textDecoder.decode(content);
                default:
                    throw new Error('Unknown message type');
            }
        }
        return content;
    };
    /**
     * Identify the user data type.
     */
    UserMessage.prototype.userDataToType = function (data) {
        if (data instanceof Uint8Array) {
            return { type: user.Message.Type.U_INT_8_ARRAY, bytes: data };
        }
        else if (typeof data === 'string') {
            return { type: user.Message.Type.STRING, bytes: textEncoder.encode(data) };
        }
        else if (data instanceof String) {
            return { type: user.Message.Type.STRING, bytes: textEncoder.encode('' + data) };
        }
        else {
            throw new Error('Message neigther a string type or a Uint8Array type');
        }
    };
    UserMessage.prototype.getBuffer = function (peerId, msgId) {
        var buffers = this.buffers.get(peerId);
        if (buffers !== undefined) {
            return buffers.get(msgId);
        }
        return undefined;
    };
    UserMessage.prototype.setBuffer = function (peerId, msgId, buffer) {
        var buffers = this.buffers.get(peerId);
        if (buffers === undefined) {
            buffers = new Map();
        }
        buffers.set(msgId, buffer);
        this.buffers.set(peerId, buffers);
    };
    return UserMessage;
}());
export { UserMessage };
/**
 * Buffer class used when the user message exceeds the message size limit which
 * may be sent over a `Channel`. Each buffer is identified by `WebChannel` id,
 * peer id of the sender and message id (in case if the peer sent more then
 * 1 big message at a time).
 */
var Buffer = /** @class */ (function () {
    function Buffer(totalLength, data, chunkNb) {
        this.fullData = new Uint8Array(totalLength);
        this.currentLength = 0;
        this.append(data, chunkNb);
    }
    /**
     * Add a chunk of message to the buffer.
     */
    Buffer.prototype.append = function (data, chunkNb) {
        var i = chunkNb * MAX_USER_MSG_SIZE;
        this.currentLength += data.length;
        try {
            for (var data_1 = __values(data), data_1_1 = data_1.next(); !data_1_1.done; data_1_1 = data_1.next()) {
                var d = data_1_1.value;
                this.fullData[i++] = d;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (data_1_1 && !data_1_1.done && (_a = data_1.return)) _a.call(data_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return this.currentLength === this.fullData.length ? this.fullData : undefined;
        var e_1, _a;
    };
    return Buffer;
}());
