import { TopologyEnum } from './service/topology/Topology';
import { IWebChannelOptions as WebGroupOptions, WebChannel, WebChannelState } from './service/WebChannel';
import { SignalingState } from './Signaling';
/**
 * Is a helper type representing types that can be sent/received over a web group.
 * @typedef {string|Uint8Array} DataType
 */
/**
 * @ignore
 */
export declare const wcs: WeakMap<WebGroup, WebChannel>;
export { WebGroupOptions };
export declare type DataType = string | Uint8Array;
/**
 * {@link WebGroup} state enum.
 */
export declare class WebGroupState {
    /**
     * Equals to `0`: joining the group...
     * @type {number}
     */
    static readonly JOINING: number;
    /**
     * Equals to `1`: joined the group successfully.
     * @type {number}
     */
    static readonly JOINED: number;
    /**
     * Equals to `2`: The `WebGroup` is in process of leaving.
     * @type {number}
     */
    static readonly LEAVING: number;
    /**
     * Equals to `3`: left the group. If the connection to the web group has lost other then
     * by calling {@link WebGroup#leave} methods and {@link WebGroup#autoRejoin} is true,
     * then the state would be `LEFT`, usually during a relatively short period) before
     * the rejoining process starts.
     * @type {number}
     */
    static readonly LEFT: number;
}
/**
 * This class is an API starting point. It represents a peer to peer network,
 * simply called a group. Each group member can send/receive broadcast
 * as well as personal messages, invite other persons or bots (see {@link WebGroupBotServer}).
 * @example
 * // Create a WebGroup with full mesh topology, autorejoin feature and
 * // specified Signaling and ICE servers for WebRTC.
 *
 * const wg = new WebGroup({
 *   signalingServer: 'wss://mysignaling.com',
 *   rtcConfiguration: {
 *     iceServers: [
 *       {
 *         urls: 'stun.l.google.com:19302'
 *       },
 *       {
 *         urls: ['turn:myturn.com?transport=udp', 'turn:myturn?transport=tcp'],
 *         username: 'user',
 *         password: 'password'
 *       }
 *     ]
 *   }
 * })
 *
 * wg.onMemberJoin = (id) => {
 *   // YOUR CODE...
 * }
 * wg.onMemberLeave = (id) => {
 *   // YOUR CODE...
 * }
 * wg.onMessage = (id, data) => {
 *   // YOUR CODE...
 * }
 * wg.onStateChange = (state) => {
 *   // YOUR CODE...
 * }
 * wg.onSignalingStateChange = (state) => {
 *   // YOUR CODE...
 * }
 */
export declare class WebGroup {
    id: number;
    myId: number;
    key: string;
    members: number[];
    topology: TopologyEnum;
    state: WebChannelState;
    signalingState: SignalingState;
    signalingServer: string;
    autoRejoin: boolean;
    onMessage: ((id: number, data: DataType) => void) | undefined | null;
    onMemberJoin: ((id: number) => void) | undefined | null;
    onMemberLeave: ((id: number) => void) | undefined | null;
    onStateChange: ((state: WebGroupState) => void) | undefined | null;
    onSignalingStateChange: ((state: SignalingState) => void) | undefined | null;
    /**
     * @param {WebGroupOptions} [options]
     * @param {Topology} [options.topology=Topology.FULL_MESH]
     * @param {string} [options.signalingServer='wss://signaling.netflux.coedit.re']
     * @param {RTCConfiguration} [options.rtcConfiguration={iceServers: [{urls: 'stun:stun3.l.google.com:19302'}]}]
     * @param {boolean} [options.autoRejoin=true]
     */
    constructor(options?: WebGroupOptions);
    /**
     * Join the group identified by a key provided by one of the group member.
     * If the current {@link WebGroup#state} value is not {@link WebGroupState#LEFT} or
     * {@link WebGroup#signalingState} value is not {@link SignalingState.CLOSED},
     * then do nothing.
     * @param {string} [key] Will be generated if not provided
     */
    join(key?: string): void;
    /**
     * Invite a bot server to join this group.
     * @param {string} url - Bot server URL (See {@link WebGroupBotServerOptions})
     */
    invite(url: string): void;
    /**
     * Leave the group which means close channels with all members and connection
     * with the Signaling server.
     */
    leave(): void;
    /**
     * Broadcast a message to the group.
     * @param {DataType} data
     */
    send(data: DataType): void;
    /**
     * Send a message to a particular group member.
     * @param {number}    id Member identifier
     * @param {DataType}  data Message
     */
    sendTo(id: number, data: DataType): void;
}
